<?php


class app extends db {


    private $db;
    private $time_start;
    public $status;
    public $priority;


    function __construct($dbhost = false, $dbuser = false, $dbpass = false, $dbname = false, $dbcharset = 'utf8') {

        $this->time_start = microtime(true);

        $this->status = [
            'open'   => 0,
            'closed' => 1
        ];

        $this->priority = [
            1 => 'niski',
            2 => 'średni',
            3 => 'wysoki'
        ];

        return $this->db = new db($dbhost, $dbuser, $dbpass, $dbname, $dbcharset);

    }


    function get_num_db_queries() {

        return $this->db->get_num_queries();

    }


    function time_from_start() {

        return microtime(true)-$this->time_start;

    }


    function get_latest($num = 10, $page = 1) {

        $count_only = ($num === COUNT) ? true : false;

        if($count_only) {
            $select = "COUNT(*) AS count";
            $orderlimit = "";
        }
        else {

            $num = intval($num);
            if($num == 0) $num = 10;
    
            if(!$page) $page = 1;
    
            $offset = 0;
            if($page > 0) $offset = ($page-1)*$num;

            $select = "*, (SELECT `time` FROM `posts` AS p WHERE p.`parent` = t.`id` OR p.`id` = t.`id` ORDER BY p.`time` DESC LIMIT 1) AS last_post_time, (SELECT `text` FROM `posts` AS p WHERE p.`parent` = t.`id` OR p.`id` = t.`id` ORDER BY p.`time` DESC LIMIT 1) AS last_post_text";
            $orderlimit =  "ORDER BY t.status ASC, t.priority DESC, last_post_time DESC LIMIT ".$offset.", ".$num;

        }

        $rows = $this->db->get_query_results("SELECT ".$select." FROM `posts` AS t WHERE t.`parent` = 0 ".$orderlimit);

        return ($count_only) ? intval($rows[0]['count']) : $rows;

    }


    function search_threads($search, $num = 10, $page = 1) {

        $count_only = ($num === COUNT) ? true : false;

        if($count_only) {
            $select = "COUNT(*) AS count";
            $orderlimit = "";
        }
        else {

            $num = intval($num);
            if($num == 0) $num = 10;
    
            if(!$page) $page = 1;
    
            $offset = 0;
            if($page > 0) $offset = ($page-1)*$num;

            $select = "*, (SELECT `time` FROM `posts` AS p WHERE p.`parent` = t.`id` OR p.`id` = t.`id` ORDER BY p.`time` DESC LIMIT 1) AS last_post_time, (SELECT `text` FROM `posts` AS p WHERE p.`parent` = t.`id` OR p.`id` = t.`id` ORDER BY p.`time` DESC LIMIT 1) AS last_post_text";
            $orderlimit =  "ORDER BY t.status ASC, t.priority DESC, last_post_time DESC LIMIT ".$offset.", ".$num;

        }

        $rows = $this->db->get_query_results("SELECT ".$select." FROM `posts` AS t WHERE t.`parent` = 0 AND (t.`id` IN (SELECT `parent` FROM `posts` WHERE (`text` LIKE '%".$this->db->escape($search)."%' OR `email` LIKE '%".$this->db->escape($search)."%') AND `parent` != 0) OR t.`id` IN (SELECT `id` FROM `posts` WHERE (`text` LIKE '%".$this->db->escape($search)."%' OR `email` LIKE '%".$this->db->escape($search)."%') AND `parent` = 0)) ".$orderlimit);

        return ($count_only) ? intval($rows[0]['count']) : $rows;

    }


    function get_thread($id) {

        $rows = $this->db->get_query_results("
        SELECT *, (SELECT GROUP_CONCAT(`file` SEPARATOR ',')
        FROM `attachments` AS a WHERE p.id = a.post
        GROUP BY a.post) AS attachments
        FROM `posts` AS p WHERE (`id` = ".intval($id)." AND `parent` = 0) OR `parent` = ".intval($id)." ORDER BY `parent` = 0 DESC, `time` ASC
        ");

        // if($rows[0]['parent'] != '0') return false;

        if($rows) {
            foreach($rows as $k => $row) {
                if($row['attachments']) $rows[$k]['attachments']  = explode(',', $row['attachments']);
                else $rows[$k]['attachments']  = false;
            }
        }

        return $rows;

    }


    function get_id_by_key($key) {

        return $this->db->get_var('posts', 'id', ['key' => $key]);

    }


    function get_key_by_id($id) {

        return $this->db->get_var('posts', 'key', ['id' => $id]);

    }


    function add($text, $email, $files = false) {

        $new_id = $this->reply(0, $text, $email, $files);
        return $new_id;

    }


    function reply($parent_id, $text, $email, $files = false) {

        $this->db->insert('posts', [
            'key'    => uniqid(),
            'parent' => $parent_id,
            'text'   => $text,
            'time'   => time(),
            'email'  => $email
        ]);

        $inserted_id = $this->db->inserted_id();

        if($files) {
            foreach($files as $file) {
                if($file) {
                    $this->db->insert('attachments', [
                        'post' => $inserted_id,
                        'file' => $file
                    ]);
                }
            }
        }

        /* notify */

        $thread_id = ($parent_id == 0) ? $inserted_id : $parent_id;

        $send_to = $this->get_all_admin_emails_to_notify(true);
        $send_to[] = $email;

        if($parent_id == 0) $mail_template = $this->get_mail_content('new_thread');
        else $mail_template = $this->get_mail_content('new_reply');

        if($send_to) {

            foreach($send_to as $to_email) {

                if(isset($to_email['email'])) $to_email = $to_email['email'];

                if($email == $to_email) continue;

                $mail = $mail_template;
                $thread_url = URL.'?action=view&amp;key='.$this->get_key_by_id($thread_id);

                if(!$this->email_is_admin($to_email)) {
                    $thread_url .= '&amp;hash='.$app->get_hash_for_email($to_email);
                }

                $thread_url .= '#last';

                $mail = str_replace('[thread_url]', $thread_url, $mail);
                $mail = str_replace('[author]', $email, $mail);
                $mail = str_replace('[text]', $text, $mail);

                $this->send_mail($to_email, DEFAULT_SEND_FROM, DEFAULT_MAIL_TITLE, $mail);

            }

        }

        return $inserted_id;

    }


    function change_priority($id, $priority) {

        if($priority < 1 || $priority > 3) $priority = 2;

        $this->db->update('posts', 
            ['priority' => $priority],
            ['id' => $id]
        );

    }


    function change_status($id, $status) {

        $this->db->update('posts', 
            ['status' => $status],
            ['id' => $id]
        );

    }


    function open($id) {

        $this->change_status($id, $this->status['open']);

    }


    function close($id) {

        $this->change_status($id, $this->status['closed']);

    }


    function upload_files($files) {

        $filenames = [];

        if($files['name']) {
            foreach($files['name'] as $k => $file) {
                $filenames[] = $this->upload_file($files['tmp_name'][$k], $files['name'][$k]);
            }
        }

        return $filenames;

    }


    function upload_file($data, $target_name = false) {

        if(!$data) return false;

        if(!$target_name) $target_name = date('Ymd').'_'.uniqid().'.png';
        $target_file = UPLOAD_PATH.$target_name;

        while(file_exists($target_file)) {

            $ext = pathinfo($target_file, PATHINFO_EXTENSION);
            $target_name = pathinfo($target_file, PATHINFO_FILENAME);

            preg_match('/_(\d+)/', $target_name, $match);
            if($match) $new_target_name = str_replace('_'.$match[1], '_'.(intval($match[1])+1), $target_name);
            else $new_target_name = $target_name.'_2';

            $target_file = str_replace($target_name, $new_target_name, $target_file);
            $target_name = pathinfo($target_file, PATHINFO_FILENAME).'.'.$ext;

        }

        // is uploaded file
        if(file_exists($data)) {
            move_uploaded_file($data, $target_file);
            return $target_name;
        }
        // is plain data
        elseif($data) {
            file_put_contents($target_file, $data);
            return $target_name;
        }

        return false;

    }


    function redirect($url, $sec = false) {

        if($sec > 0) {
            header('Refresh: '.intval($sec).'; URL='.$url);
        }
        else {
            header('Location: '.$url);
            exit;
        }

    }


    function get_hash_for_email($email) {

        return md5($email.EMAIL_SALT);

    }

    
    function get_hash_by_id($id) {

        return $this->get_hash_for_email($this->db->get_var('posts', 'email', ['id' => $id]));

    }


    function can_view($post) {

        if(isset($_SESSION['admin'])) return true; // to do
        elseif(isset($_GET['hash'])) return ($this->get_hash_for_email($post['email']) == $_GET['hash']);

        return false;

    }


    function login($email, $password) {

        $row = $this->db->get_result('admins', ['email' => $email]);

        if(!$row) return false;

        $hashed = md5($password.$row['salt']);

        if($row['password'] == $hashed) {
            $_SESSION['admin'] = $row['email'];
            return true;
        }
        return false;

    }


    function is_logged() {

        return isset($_SESSION['admin']);

    }


    function email_is_admin($email) {

        return boolval($this->db->get_var('admins', 'email', ['email' => $email]));

    }


    function cut_text($text, $n = 200) {

		$text = trim($text);
		$text = strip_tags($text);
		$org = strlen($text);

		if($org > $n) {
			$text = mb_substr($text, 0, $n).' ';
			$text = preg_replace('#&([a-z]+) #', '', $text);
			$text = trim($text);
			$text .= '...';
		}

		$text = preg_replace('/\s\s+/', ' ', $text);
		$text = str_replace('&nbsp; ', '', $text);
		$text = str_replace(' &nbsp;', '', $text);

		$text = str_replace('</td><td>', '</td> <td>', $text);
		$text = str_replace('</li>', '</li> ', $text);
		$text = str_replace("\n", ' ', $text);
		$text = str_replace("\r", ' ', $text);

		$text = str_replace('.&nbsp;...', '...', $text);
		$text = str_replace('....', '...', $text);
		$text = str_replace('&...', '...', $text);

		if(substr($text, 0, 6) == '&nbsp;') $text = substr($text, 6);
		
		if($text == '...') return false;
		else return $text;

    }
    

    function get_thread_autologin_url($id) {

        $key = $app->get_key_by_id($id);
        $url = URL.'?action=view&key='.$key;
        $url .= '&hash='.$app->get_hash_by_id($id);

        return $url;

    }


    function get_all_admin_emails_to_notify($not_me = false) {

        $where = [];

        $where[] = "`notify` = 1";
        if($not_me && isset($_SESSION['admin'])) $where[] = "`email` != '".$this->db->escape($_SESSION['admin'])."'";

        if($where) $where = implode(" AND ", $where);

        $rows = $this->db->get_results('admins', $where, '', 0, 0, ['email']);

        return $rows;

    }


    function get_mail_content($name) {

        $file = PATH.'templates/mails/'.$name.'.html';
        return (file_exists($file)) ? file_get_contents($file) : '';

    }


    function send_mail($to, $from, $subject, $text, $files = false) {

        // to do - choose method, default: SMTP
        return $this->mail_by_smtp($to, $from, $subject, $text, $files);

    }


    function mail_by_smtp($to, $from, $subject, $text, $files = false, $debug = false) {

        // $app->send_mail('f.klar@wobit.com.pl', false, 'Pozdrowienia', 'Lorem ipsum', false, true); // example

        require_once(PATH.'inc/swiftmailer/swift_required.php');

        $transport = Swift_SmtpTransport::newInstance(SMTP_HOST, SMTP_PORT)
        ->setUsername(SMTP_EMAIL)
        ->setPassword(SMTP_PASS);
        $mailer = Swift_Mailer::newInstance($transport);
        $message = Swift_Message::newInstance();

        $mime = (strlen(strip_tags($text)) == strlen($text)) ? 'text/plain' : 'text/html';

        if($mime == 'text/html') {
            $text = str_replace('<blockquote>', '<blockquote style="font-style:italic">', $text);
        }

        $message->setCharset('utf-8');
        if($subject) $message->setSubject(strip_tags($subject));
        if($text) $message->setBody($text, $mime);

        if($debug) var_dump($text, $message);

        if($files) {
            foreach($files as $file) {
                if(is_array($file)) {
                    $message->attach(Swift_Attachment::fromPath($file['path'])->setFilename($file['name']));
                }
                else $message->attach(Swift_Attachment::fromPath($file));
            }
        }

        if($from) $message->setFrom($from);
        else $message->setFrom(SMTP_EMAIL);
        if($from) $message->setReplyTo($from);

        $message->setTo($to);

        $email_sent = $mailer->send($message, $failed);

        return $email_sent;

    }


    function tpl_escape($str) {

        $str = strip_tags($str);
        $str = addslashes($str);
        return $str;

    }


    function get_current_url() {

        return URL.str_replace('/?', '?', $_SERVER['REQUEST_URI']);;

    }


    function change_url_page($new_page) {

        $url = $this->get_current_url();
        if(strpos($url, 'page=') !== false) $url = preg_replace('/page=(\d+)/', 'page='.$new_page, $url);
        else $url .= '&page='.$new_page;

        $url = str_replace('&page=1', '', $url);

        return $url;

    }


}
